@@ -9,35 +9,47 @@ from webob import UnicodeMultiDict
 
 import schemaish, validatish
 from formish import util
-from dottedish import dotted, is_int
+from dottedish import dotted, unflatten, set
 from formish import validation
 from formish import widgets
 from formish.renderer import _default_renderer
 
 NOARG = object()
 
+def container_factory(parent_key, item_key):
+    if item_key.isdigit():
+        return []
+    return {}
+
+def is_int(v):
+    """ raise error if not """
+    try:
+        int(v)
+        return True
+    except ValueError:
+        return False
+
 class Action(object):
     """
     An action that that can added to a form.
 
-    :arg callback: A callable with the signature (request, form, *args)
     :arg name: an valid html id used to lookup an action
-    :arg label: The 'value' of the submit button and hence the text that people see
+    :arg value: The 'value' of the submit button and hence the text that people see
+    :arg callback: A callable with the signature (request, form, *args)
     """
-    def __init__(self, callback, name, label):
-        if not util.valid_identifier(name):
+    def __init__(self, name=None, value=None, callback=None):
+        if name and not util.valid_identifier(name):
             raise validation.FormError('Invalid action name %r.'% name)
         self.callback = callback
         self.name = name
-        if label is None:
-            self.label = util.title_from_name(name)
-        else:
-            self.label = label
+        self.value = value
 
             
 def _cssname(self):
     """ Returns a hyphenated identifier using the form name and field name """
-    return '%s-%s'% (self.form.name, '-'.join(self.name.split('.')))    
+    if self.form.name:
+        return '%s-%s'% (self.form.name, '-'.join(self.name.split('.')))    
+    return '%s'% ('-'.join(self.name.split('.')))    
 
 
 def _classes(self):
@@ -74,6 +86,12 @@ def starify(name):
 
 
 def fall_back_renderer(renderer, name, widget, vars):
+    """
+    Tries to find template in widget directly then tries in top level directory
+    
+    This allows a field level widget override it's container by including the
+    changed version in the widgets directory with the same name
+    """
     import mako
     try:
         return renderer('/formish/widgets/%s/%s.html'%(widget,name), vars)
@@ -138,6 +156,7 @@ class Field(object):
         :type form: formish.Form instance.
         """
         self.name = name
+        self.nodename = name.split('.')[-1]
         self.attr = attr
         self.form = form
 
@@ -146,7 +165,7 @@ class Field(object):
 
     @property
     def title(self):
-        """ The Field schema's title """
+        """ The Field schema's title - derived from name if not specified """
         try:
             return self.form.get_item_data(self.name,'title')
         except KeyError:
@@ -182,7 +201,7 @@ class Field(object):
     def value(self):
         """Convert the request_data to a value object for the form or None."""
         if '*' in self.name:
-            return  self.widget.to_request_data(self.attr, self.defaults)
+            return  self.widget.to_request_data(self, self.defaults)
         return self.form.request_data.get(self.name, None)
 
 
@@ -211,11 +230,16 @@ class Field(object):
             val = ''
         return TemplatedString(self, 'error', val)
 
-    @property
-    def errors(self):
+    
+    def _get_errors(self):
         """ Lazily get the error from the form.errors when needed """
         return self.form.errors.get(self.name, None)
 
+    def _set_errors(self, v):
+        self.form.errors[self.name] = v
+
+    errors = property(_get_errors, _set_errors)
+
     @property
     def widget(self):
         """ return the fields widget bound with extra params. """
@@ -224,6 +248,7 @@ class Field(object):
             widget_type = self.form.get_item_data(starify(self.name),'widget')
         except KeyError:
             widget_type = widgets.Input()
+            self.form.set_item_data(starify(self.name),'widget',widget_type)
         return BoundWidget(widget_type, self)
 
         
@@ -291,6 +316,10 @@ class Collection(object):
         :type form: formish.Form instance.
         """
         self.name = name
+        if name is not None:
+            self.nodename = name.split('.')[-1]
+        else:
+            self.nodename = ''
         self.attr = attr
         self.form = form
         self._fields = {}    
@@ -356,11 +385,15 @@ class Collection(object):
         val = self.form.errors.get(self.name, None)
         return TemplatedString(self, 'error', val)
 
-    @property
-    def errors(self):
+    def _get_errors(self):
         """ Lazily get the error from the form.errors when needed """
         return self.form.errors.get(self.name, None)
 
+    def _set_errors(self, v):
+        self.form.errors[self.name] = v
+
+    errors = property(_get_errors, _set_errors)
+
     @property
     def contains_error(self):
         """ Check to see if any child elements have errors """
@@ -376,7 +409,11 @@ class Collection(object):
         try:
             widget_type = BoundWidget(self.form.get_item_data(starify(self.name),'widget'),self)
         except KeyError:
-            widget_type = BoundWidget(widgets.SequenceDefault(),self)
+            if self.type == 'group':
+                widget_type = BoundWidget(widgets.StructureDefault(),self)
+            else:
+                widget_type = BoundWidget(widgets.SequenceDefault(),self)
+            self.form.set_item_data(starify(self.name),'widget',widget_type)
         return widget_type
 
 
@@ -506,26 +543,32 @@ class Sequence(Collection):
         # request_data we need to recurse throught the fields ... which calls
         # Sequence.fields ... which tries to build the request data ... which
         # calls Sequence.fields, etc, etc. Bang!
-        if self.form._request_data:
-            data = self.form._request_data.get(self.name, [])
-        else:
-            data = self.defaults
 
-        if data is None:
-            num_fields = 0
-        else:
-            num_fields = len(data)
-            
-        minimum = None
-        maximum = None
-        if self.widget is not None:
-            minimum = getattr(self.widget, 'min', None)
-            maximum = getattr(self.widget, 'max', None)
-        if minimum is not None and num_fields < min:
-            num_fields = minimum
-        elif max is not None and num_fields > max:
-            num_fields = maximum
+        if not self.form._request_data:
+            if self.defaults is not None:
+                num_fields = len(self.defaults)
+            else:
+                num_fields = 0
 
+            num_nonempty_fields = 0
+            if self.widget is not None:
+                empty_checker = self.widget.empty_checker
+            if self.defaults is not None:
+                for n,d in enumerate(self.defaults):
+                    if not empty_checker(d):
+                        num_nonempty_fields=n+1
+                
+            min_start_fields = None
+            min_empty_start_fields = None
+            if self.widget is not None:
+                min_start_fields = getattr(self.widget, 'min_start_fields', None)
+                min_empty_start_fields = getattr(self.widget, 'min_empty_start_fields', None)
+            if min_start_fields is not None and num_fields < min_start_fields:
+                num_fields = min_start_fields
+            if min_empty_start_fields is not None and (num_fields - num_nonempty_fields) < min_empty_start_fields:
+                num_fields = num_nonempty_fields + min_empty_start_fields
+        else:
+            num_fields = len(self.form._request_data.get(self.name, []))
 
         for num in xrange(num_fields):
             field = self.bind(num, self.attr.attr)
@@ -555,7 +598,11 @@ class BoundWidget(object):
 
     def __call__(self):
         widget_type, widget = self.widget.template.split('.')
-        return self.field.form.renderer('/formish/widgets/%s/widget.html'%widget, {'field':self.field})
+        if self.widget.readonly == True:
+            widget_template = 'readonly'
+        else:
+            widget_template = 'widget'
+        return self.field.form.renderer('/formish/widgets/%s/%s.html'%(widget, widget_template), {'field':self.field})
 
     def __repr__(self):
         return 'BoundWidget(widget=%r, field=%r)'%(self.widget, self.field)
@@ -591,7 +638,7 @@ class Form(object):
     _request_data = None
 
     def __init__(self, structure, name=None, defaults=None, errors=None,
-            action_url=None, renderer=None, method='POST'):
+                 action_url=None, renderer=None, method='POST', add_default_action=True, include_charset=True):
         """
         Create a new form instance
 
@@ -629,22 +676,26 @@ class Form(object):
         if errors is None:
             errors = {}
         self.defaults = defaults
-        self.errors = dotted(errors)
+        self.errors = errors
         self.error = None
         self._actions = []
+        if add_default_action:
+            self.add_action( None, 'Submit' )
         self.action_url = action_url
         if renderer is not None:
             self.renderer = renderer
         self.method = method
+        self.widget = widgets.StructureDefault()
+        self.include_charset = include_charset
 
     def __repr__(self):
         attributes = []
         attributes.append('%r'%self.structure.attr)
         attributes.append('name=%r'%self.name)
-        if self.defaults.data != {}:
-            attributes.append('defaults=%r'%self.defaults.data)
-        if self.errors.data != {}:
-            attributes.append('errors=%r'%self.errors.data)
+        if self.defaults._o != {}:
+            attributes.append('defaults=%r'%self.defaults._o)
+        if self.errors != {}:
+            attributes.append('errors=%r'%self.errors)
         if self.action_url:
             attributes.append('action_url=%r'%self.action_url)
         return 'formish.Form(%s)'%( ', '.join(attributes) )
@@ -661,23 +712,7 @@ class Form(object):
 
     element_name = property(_element_name_get, _element_name_set)
 
-    def _name_get(self):
-        """ Get the name of the form, default to `formish` """
-        if self._element_name is not None:
-            return self._element_name
-        if self._name is not None:
-            return self._name
-        return 'form'
-
-    def _name_set(self, name):
-        """ Set the form name """ 
-        if self._element_name is not None:
-            raise Exception("Named forms cannot be used as elements.")
-        self._name = name
-
-    name = property(_name_get, _name_set)
-
-    def add_action(self, callback, name="submit", label=None):
+    def add_action(self, name=None, value=None, callback=None):
         """ 
         Add an action callable to the form
 
@@ -692,9 +727,9 @@ class Form(object):
         :type label: string
         
         """
-        if name in [action.name for action in self._actions]:
+        if name and name in [action.name for action in self._actions]:
             raise ValueError('Action with name %r already exists.'% name)
-        self._actions.append( Action(callback, name, label) )              
+        self._actions.append( Action(name, value, callback) )              
 
     def action(self, request, *args):
         """
@@ -715,7 +750,7 @@ class Form(object):
         return self._actions[0].callback(request, self, *args)
 
 
-    def get_unvalidated_data(self, request_data, raise_exceptions=True):
+    def get_unvalidated_data(self, request_data, raise_exceptions=True, skip_read_only_defaults=False):
         """
         Convert the request object into a nested dict in the correct structure
         of the schema but without applying the schema's validation.
@@ -723,11 +758,11 @@ class Form(object):
         :arg request_data: Webob style request data
         :arg raise_exceptions: Whether to raise exceptions or return errors
         """
-        data = validation.from_request_data(self.structure, request_data, errors=self.errors) 
+        data = self.widget.from_request_data(self.structure, request_data, skip_read_only_defaults=skip_read_only_defaults) 
         if raise_exceptions and len(self.errors.keys()):
             raise validation.FormError( \
         'Tried to access data but conversion from request failed with %s errors (%s)'% \
-                   (len(self.errors.keys()), self.errors.data))
+                   (len(self.errors.keys()), self.errors))
         return data
     
 
@@ -737,9 +772,9 @@ class Form(object):
         request_data format.
         """
         if self._request_data is not None:
-            return self._request_data
-        self._request_data = validation.to_request_data(self.structure, dotted(self.defaults))
-        return self._request_data
+            return dotted(self._request_data)
+        self._request_data = self.widget.to_request_data(self.structure, self._defaults)
+        return dotted(self._request_data)
 
 
     def _set_request_data(self, request_data):
@@ -749,7 +784,7 @@ class Form(object):
         :arg request_data: raw request data (e.g. request.POST)
         :type request_data: Dictionary (dotted or nested or dotted or MultiDict)
         """
-        self._request_data = dotted(request_data)
+        self._request_data = request_data
 
 
     request_data = property(_get_request_data, _set_request_data)
@@ -767,9 +802,42 @@ class Form(object):
    
 
     defaults = property(_get_defaults, _set_defaults)
-    
 
-    def validate(self, request, failure_callable=None, success_callable=None):
+    def _set_request(self, request):
+        """ 
+        Assign raw request data to the form
+        
+        :arg request_data: raw request data (e.g. request.POST)
+        :type request_data: Dictionary (dotted or nested or dotted or MultiDict)
+        """
+        self._request = request
+        request_data = getattr(request, self.method.upper())
+        # Decode request data according to the request's charset.
+        request_data = UnicodeMultiDict(request_data,
+                                        encoding=util.get_post_charset(request))
+        # Remove the sequence factory data from the request
+        for k in request_data.keys():
+            if '*' in k:
+                request_data.pop(k)
+        # We need the _request_data to be populated so sequences know how many
+        # items they have (i.e. .fields method on a sequence uses the number of
+        # values on the _request_data)
+
+        # Convert request data to a dottedish friendly representation
+        request_data = _unflatten_request_data(request_data)
+        self._request_data = request_data
+        self._request_data = self.widget.pre_parse_incoming_request_data(self.structure,request_data)
+
+    def _get_request(self):
+        return self._request
+
+    request = property(_get_request, _set_request)
+
+    def name_from_request(self, request):
+        request_data = getattr(request, self.method.upper())
+        return request_data.get('__formish_form__')
+
+    def validate(self, request, failure_callable=None, success_callable=None, skip_read_only_defaults=False, check_form_name=True):
         """ 
         Validate the form data in the request.
 
@@ -785,8 +853,12 @@ class Form(object):
         :returns: Python dict of converted and validated data.
         :raises: formish.FormError, raised on validation failure.
         """
+        # Check this request was submitted by this form.
+        self.request = request
+        if check_form_name == True and (self.name != self.name_from_request(request)):
+            raise Exception("request does not match form name")
         try: 
-            data = self._validate(request)
+            data = self._validate(request, skip_read_only_defaults=skip_read_only_defaults)
         except validation.FormError, e:
             if failure_callable is None:
                 raise
@@ -797,35 +869,14 @@ class Form(object):
         else:
             return success_callable(request, data)
 
-    def _validate(self, request):
+    def _validate(self, request, skip_read_only_defaults=False):
         """
         Get the data without raising exceptions and then validate the data. If
         there are errors, raise them; otherwise return the data
         """
         # XXX Should this happen after the basic stuff has happened?
         self.errors = {}
-        # Decide what request data to use. The method will have been validated
-        # already (unless someone's messing around!) so we can just use it in a
-        # getattr.
-        request_data = getattr(request, self.method.upper())
-        # Check this request was submitted by this form.
-        if request_data.get('__formish_form__') != self.name:
-            raise Exception("request does not match form name")
-        # Decode request data according to the request's charset.
-        request_data = UnicodeMultiDict(request_data,
-                                        encoding=util.get_post_charset(request))
-        # Remove the sequence factory data from the request
-        for k in request_data.keys():
-            if '*' in k:
-                request_data.pop(k)
-        # We need the _request_data to be populated so sequences know how many
-        # items they have (i.e. .fields method on a sequence uses the number of
-        # values on the _request_data)
-        self._request_data = dotted(request_data)
-        self.request_data = validation.pre_parse_incoming_request_data( \
-                    self.structure,dotted(request_data))
-        data = self.get_unvalidated_data( \
-                    self.request_data, raise_exceptions=False)
+        data = self.get_unvalidated_data(self._request_data, raise_exceptions=False, skip_read_only_defaults=skip_read_only_defaults)
         try:
             self.structure.attr.validate(data)
         except schemaish.attr.Invalid, e:
@@ -845,7 +896,7 @@ class Form(object):
         allowed = ['title', 'widget', 'description','default']
         if name in allowed:
             if name == 'default' and '*' not in key:
-                self.defaults[key] = value
+                set(self.defaults,key,value,container_factory=container_factory)
             else:
                 self.item_data.setdefault(key, {})[name] = value
         else:
@@ -926,12 +977,29 @@ class Form(object):
         """ Return just the metada part of the template """
         return self.renderer('/formish/form/metadata.html', {'form':self})
 
+    def error_list(self):
+        """ Return just the metada part of the template """
+        return self.renderer('/formish/form/error_list.html', {'form':self})
+
     def actions(self):
         """ Return just the actions part of the template """
         return self.renderer('/formish/form/actions.html', {'form':self})
 
         
-   
+def _unflatten_request_data(request_data):
+    """
+    Unflatten the request data into nested dicts and lists.
+    """
+    # Build an ordered list of keys. Don't rely on the request_data doing this
+    # for us because webob's MultiDict yields the same key multiple times!
+    # Of course, if request_data is not an ordered dict then this is fairly
+    # pointless anyway.
+    keys = []
+    for key in request_data:
+        if key not in keys:
+            keys.append(key)
+    return unflatten(((key, request_data.getall(key)) for key in keys),
+                     container_factory=container_factory) 
 
 
 class FormAccessor(object):
