@@ -1,6 +1,6 @@
 import logging, os.path, subprocess
 import formish, schemaish, validatish
-from dottedish import dotted
+from dottedish.api import dotted, flatten
 from formish.filestore import CachedTempFilestore
 from testish.lib import xformish
 import webob
@@ -24,7 +24,7 @@ def build_request(formname, data, rawdata=False):
             fields.append(d) 
     else:
         d = dotted(data)
-        for k, v in d.dotteditems():
+        for k, v in flatten(d):
             fields.append( (k,v) )
     fields.append( ('submit','Submit') )
     request.body = urlencode( fields )
@@ -53,22 +53,19 @@ def functest_String(self):
     sel = self.selenium
     sel.open("/String")
 
-    sel.click("form-action-submit")
+    sel.click("form-action")
     sel.wait_for_page_to_load("30000")
-    try: self.failUnless(sel.is_text_present("{'myStringField': None}"))
-    except AssertionError, e: self.verificationErrors.append(str(e))
+    self.failUnless(sel.is_text_present("{'myStringField': None}"))
 
     sel.type("form-myStringField", "Test")
-    sel.click("form-action-submit")
+    sel.click("form-action")
     sel.wait_for_page_to_load("30000")
-    try: self.failUnless(sel.is_text_present("{'myStringField': u'Test'}"))
-    except AssertionError, e: self.verificationErrors.append(str(e))
+    self.failUnless(sel.is_text_present("{'myStringField': u'Test'}"))
 
     sel.type("form-myStringField", "80")
-    sel.click("form-action-submit")
+    sel.click("form-action")
     sel.wait_for_page_to_load("30000")
-    try: self.failUnless(sel.is_text_present("{'myStringField': u'80'}"))
-    except AssertionError, e: self.verificationErrors.append(str(e))
+    self.failUnless(sel.is_text_present("{'myStringField': u'80'}"))
 
     return
 
@@ -135,28 +132,24 @@ def functest_Integer(self):
     sel = self.selenium
     sel.open("/Integer")
 
-    sel.click("form-action-submit")
+    sel.click("form-action")
     sel.wait_for_page_to_load("30000")
-    try: self.failUnless(sel.is_text_present("{'myIntegerField': None}"))
-    except AssertionError, e: self.verificationErrors.append(str(e))
+    self.failUnless(sel.is_text_present("{'myIntegerField': None}"))
 
     sel.type("form-myIntegerField", "a")
-    sel.click("form-action-submit")
+    sel.click("form-action")
     sel.wait_for_page_to_load("30000")
-    try: self.failUnless(sel.is_text_present("Not a valid integer"))
-    except AssertionError, e: self.verificationErrors.append(str(e))
+    self.failUnless(sel.is_text_present("Not a valid integer"))
 
     sel.type("form-myIntegerField", "8.0")
-    sel.click("form-action-submit")
+    sel.click("form-action")
     sel.wait_for_page_to_load("30000")
-    try: self.failUnless(sel.is_text_present("Not a valid integer"))
-    except AssertionError, e: self.verificationErrors.append(str(e))
+    self.failUnless(sel.is_text_present("Not a valid integer"))
 
     sel.type("form-myIntegerField", "8")
-    sel.click("form-action-submit")
+    sel.click("form-action")
     sel.wait_for_page_to_load("30000")
-    try: self.failUnless(sel.is_text_present("{'myIntegerField': 8}"))
-    except AssertionError, e: self.verificationErrors.append(str(e))
+    self.failUnless(sel.is_text_present("{'myIntegerField': 8}"))
 
     return
 
@@ -201,28 +194,24 @@ def functest_Date(self):
 
     sel.open("/Date")
 
-    sel.click("form-action-submit")
+    sel.click("form-action")
     sel.wait_for_page_to_load("30000")
-    try: self.failUnless(sel.is_text_present("{'myDateField': None}"))
-    except AssertionError, e: self.verificationErrors.append(str(e))
+    self.failUnless(sel.is_text_present("{'myDateField': None}"))
 
     sel.type("form-myDateField", "a")
-    sel.click("form-action-submit")
+    sel.click("form-action")
     sel.wait_for_page_to_load("30000")
-    try: self.failUnless(sel.is_text_present("Invalid date"))
-    except AssertionError, e: self.verificationErrors.append(str(e))
+    self.failUnless(sel.is_text_present("Invalid date"))
 
     sel.type("form-myDateField", "18/12/1966")
-    sel.click("form-action-submit")
+    sel.click("form-action")
     sel.wait_for_page_to_load("30000")
-    try: self.failUnless(sel.is_text_present("Invalid date"))
-    except AssertionError, e: self.verificationErrors.append(str(e))
+    self.failUnless(sel.is_text_present("Invalid date"))
 
     sel.type("form-myDateField", "2008-12-18")
-    sel.click("form-action-submit")
+    sel.click("form-action")
     sel.wait_for_page_to_load("30000")
-    try: self.failUnless(sel.is_text_present("{'myDateField': datetime.date(2008, 12, 18)}"))
-    except AssertionError, e: self.verificationErrors.append(str(e))
+    self.failUnless(sel.is_text_present("{'myDateField': datetime.date(2008, 12, 18)}"))
 
     return
 
@@ -259,31 +248,43 @@ def functest_Float(self):
     sel = self.selenium
     sel.open("/Float")
 
-    sel.click("form-action-submit")
+    sel.click("form-action")
     sel.wait_for_page_to_load("30000")
-    try: self.failUnless(sel.is_text_present("{'myFloatField': None}"))
-    except AssertionError, e: self.verificationErrors.append(str(e))
+    self.failUnless(sel.is_text_present("{'myFloatField': None}"))
 
     sel.type("form-myFloatField", "a")
-    sel.click("form-action-submit")
+    sel.click("form-action")
     sel.wait_for_page_to_load("30000")
-    try: self.failUnless(sel.is_text_present("Not a valid number"))
-    except AssertionError, e: self.verificationErrors.append(str(e))
+    self.failUnless(sel.is_text_present("Not a valid number"))
 
     sel.type("form-myFloatField", "11")
-    sel.click("form-action-submit")
+    sel.click("form-action")
     sel.wait_for_page_to_load("30000")
-    try: self.failUnless(sel.is_text_present("{'myFloatField': 11.0}"))
-    except AssertionError, e: self.verificationErrors.append(str(e))
+    self.failUnless(sel.is_text_present("{'myFloatField': 11.0}"))
 
     sel.type("form-myFloatField", "12.27")
-    sel.click("form-action-submit")
+    sel.click("form-action")
     sel.wait_for_page_to_load("30000")
-    try: self.failUnless(sel.is_text_present("{'myFloatField': 12.27}"))
-    except AssertionError, e: self.verificationErrors.append(str(e))
+    self.failUnless(sel.is_text_present("{'myFloatField': 12.27}"))
 
     return
 
+def unittest_Float(self, formdef):
+    # Test no data
+    f =  formdef(None)
+    self.assertIdHasValue(f, 'form-myFloatField', '')
+    # Test None data
+    f = formdef(None)
+    testdata = {'myFloatField': None}
+    f.defaults = testdata
+    self.assertIdHasValue(f, 'form-myFloatField', '')
+    self.assertRoundTrip(f, testdata)
+    # Test sample data
+    f = formdef(None)
+    testdata = {'myFloatField': 8.4}
+    f.defaults = testdata
+    self.assertIdHasValue(f, 'form-myFloatField', '8.4')
+    self.assertRoundTrip(f, testdata)
 
 
 def form_Boolean(request):
@@ -299,37 +300,49 @@ def functest_Boolean(self):
     sel = self.selenium
     sel.open("/Boolean")
 
-    sel.click("form-action-submit")
+    sel.click("form-action")
     sel.wait_for_page_to_load("30000")
-    try: self.failUnless(sel.is_text_present("{'myBooleanField': None}"))
-    except AssertionError, e: self.verificationErrors.append(str(e))
+    self.failUnless(sel.is_text_present("{'myBooleanField': None}"))
 
     sel.type("form-myBooleanField", "a")
-    sel.click("form-action-submit")
+    sel.click("form-action")
     sel.wait_for_page_to_load("30000")
-    try: self.failUnless(sel.is_text_present("u'a' should be either True or False"))
-    except AssertionError, e: self.verificationErrors.append(str(e))
+    self.failUnless(sel.is_text_present("u'a' should be either True or False"))
 
     sel.type("form-myBooleanField", "t")
-    sel.click("form-action-submit")
+    sel.click("form-action")
     sel.wait_for_page_to_load("30000")
-    try: self.failUnless(sel.is_text_present("u't' should be either True or False"))
-    except AssertionError, e: self.verificationErrors.append(str(e))
+    self.failUnless(sel.is_text_present("u't' should be either True or False"))
 
     sel.type("form-myBooleanField", "True")
-    sel.click("form-action-submit")
+    sel.click("form-action")
     sel.wait_for_page_to_load("30000")
-    try: self.failUnless(sel.is_text_present("{'myBooleanField': True}"))
-    except AssertionError, e: self.verificationErrors.append(str(e))
+    self.failUnless(sel.is_text_present("{'myBooleanField': True}"))
 
     sel.type("form-myBooleanField", "False")
-    sel.click("form-action-submit")
+    sel.click("form-action")
     sel.wait_for_page_to_load("30000")
-    try: self.failUnless(sel.is_text_present("{'myBooleanField': False}"))
-    except AssertionError, e: self.verificationErrors.append(str(e))
+    self.failUnless(sel.is_text_present("{'myBooleanField': False}"))
 
     return
 
+def unittest_Boolean(self, formdef):
+    # Test no data
+    f =  formdef(None)
+    self.assertIdHasValue(f, 'form-myBooleanField', '')
+    # Test None data
+    f = formdef(None)
+    testdata = {'myBooleanField': None}
+    f.defaults = testdata
+    self.assertIdHasValue(f, 'form-myBooleanField', '')
+    self.assertRoundTrip(f, testdata)
+    # Test sample data
+    f = formdef(None)
+    testdata = {'myBooleanField': True}
+    f.defaults = testdata
+    self.assertIdHasValue(f, 'form-myBooleanField', 'True')
+    self.assertRoundTrip(f, testdata)
+
 def form_BooleanWithDefaults(request):
     """
     A simple form with a single boolean field
@@ -356,28 +369,24 @@ def functest_Decimal(self):
     sel = self.selenium
     sel.open("/Decimal")
 
-    sel.click("form-action-submit")
+    sel.click("form-action")
     sel.wait_for_page_to_load("30000")
-    try: self.failUnless(sel.is_text_present("{'myDecimalField': None}"))
-    except AssertionError, e: self.verificationErrors.append(str(e))
+    self.failUnless(sel.is_text_present("{'myDecimalField': None}"))
 
     sel.type("form-myDecimalField", "a")
-    sel.click("form-action-submit")
+    sel.click("form-action")
     sel.wait_for_page_to_load("30000")
-    try: self.failUnless(sel.is_text_present("Not a valid number"))
-    except AssertionError, e: self.verificationErrors.append(str(e))
+    self.failUnless(sel.is_text_present("Not a valid number"))
 
     sel.type("form-myDecimalField", "1")
-    sel.click("form-action-submit")
+    sel.click("form-action")
     sel.wait_for_page_to_load("30000")
-    try: self.failUnless(sel.is_text_present("{'myDecimalField': Decimal('1')}"))
-    except AssertionError, e: self.verificationErrors.append(str(e))
+    self.failUnless(sel.is_text_present("{'myDecimalField': Decimal('1')}"))
 
     sel.type("form-myDecimalField", "18.001")
-    sel.click("form-action-submit")
+    sel.click("form-action")
     sel.wait_for_page_to_load("30000")
-    try: self.failUnless(sel.is_text_present("{'myDecimalField': Decimal('18.001')}"))
-    except AssertionError, e: self.verificationErrors.append(str(e))
+    self.failUnless(sel.is_text_present("{'myDecimalField': Decimal('18.001')}"))
 
     return
 
@@ -392,27 +401,33 @@ def form_RequiredStringAndCheckbox(request):
     form['myBoolean'].widget=formish.Checkbox()
     return form
 
+def unittest_RequiredStringAndCheckbox(self, formdef):
+    # Test no data
+    f =  formdef(None)
+    self.assertIdAttrHasValue(f, 'form-myBoolean','value','True')
+    self.assertRaisesValidationError(f)
+    testdata = {'myString':'7', 'myBoolean': False}
+    f.defaults = testdata
+    self.assertRoundTrip(f, testdata)
+
 def functest_RequiredStringAndCheckbox(self):
     sel = self.selenium
     sel.open("/RequiredStringAndCheckbox")
 
-    sel.click("form-action-submit")
+    sel.click("form-action")
     sel.wait_for_page_to_load("30000")
-    try: self.failUnless(sel.is_text_present("is required"))
-    except AssertionError, e: self.verificationErrors.append(str(e))
+    self.failUnless(sel.is_text_present("is required"))
 
     sel.type("form-myString", "anything")
-    sel.click("form-action-submit")
+    sel.click("form-action")
     sel.wait_for_page_to_load("30000")
-    try: self.failUnless(sel.is_text_present("{'myBoolean': False, 'myString': u'anything'}"))
-    except AssertionError, e: self.verificationErrors.append(str(e))
+    self.failUnless(sel.is_text_present("{'myBoolean': False, 'myString': u'anything'}"))
 
     sel.type("form-myString", "anythingelse")
     sel.click("form-myBoolean")
-    sel.click("form-action-submit")
+    sel.click("form-action")
     sel.wait_for_page_to_load("30000")
-    try: self.failUnless(sel.is_text_present("{'myBoolean': True, 'myString': u'anythingelse'}"))
-    except AssertionError, e: self.verificationErrors.append(str(e))
+    self.failUnless(sel.is_text_present("{'myBoolean': True, 'myString': u'anythingelse'}"))
 
     return
 
@@ -430,18 +445,16 @@ def functest_File(self):
     sel = self.selenium
     sel.open("/File")
 
-    sel.click("form-action-submit")
+    sel.click("form-action")
     sel.wait_for_page_to_load("30000")
-    try: self.failUnless(sel.is_text_present("{'myFile': None}"))
-    except AssertionError, e: self.verificationErrors.append(str(e))
+    self.failUnless(sel.is_text_present("{'myFile': None}"))
 
     sel.type("form-myFile", os.path.abspath("testdata/test.txt"))
-    sel.click("form-action-submit")
+    sel.click("form-action")
     sel.wait_for_page_to_load("30000")
     
 
-    try: self.failUnless(sel.is_text_present("{'myFile': <schemaish.type.File"))
-    except AssertionError, e: self.verificationErrors.append(str(e))
+    self.failUnless(sel.is_text_present("{'myFile': <schemaish.type.File"))
 
     return
 
@@ -748,44 +761,36 @@ def functest_RequiredStringAndFile(self):
     sel = self.selenium
     sel.open("/RequiredStringAndFile")
 
-    sel.click("form-action-submit")
+    sel.click("form-action")
     sel.wait_for_page_to_load("30000")
         
-    try: self.assertEqual("", sel.get_value("myFileField.default"))
-    except AssertionError, e: self.verificationErrors.append(str(e))
+    self.assertEqual("", sel.get_value("myFileField.default"))
 
-    try: self.assertEqual("", sel.get_value("myFileField.name"))
-    except AssertionError, e: self.verificationErrors.append(str(e))
+    self.assertEqual("", sel.get_value("myFileField.name"))
 
 
     sel.type("form-myFileField", os.path.abspath("testdata/photo.png"))
-    sel.click("form-action-submit")
+    sel.click("form-action")
     sel.wait_for_page_to_load("30000")
         
-    try: self.assertEqual("", sel.get_value("myFileField.default"))
-    except AssertionError, e: self.verificationErrors.append(str(e))
+    self.assertEqual("", sel.get_value("myFileField.default"))
 
-    try: self.assertEqual("-photo.png", sel.get_value("myFileField.name")[-10:])
-    except AssertionError, e: self.verificationErrors.append(str(e))
+    self.assertTrue(sel.get_value("myFileField.name"))
 
-    sel.click("form-action-submit")
+    sel.click("form-action")
     sel.wait_for_page_to_load("30000")
         
-    try: self.assertEqual("", sel.get_value("myFileField.default"))
-    except AssertionError, e: self.verificationErrors.append(str(e))
+    self.assertEqual("", sel.get_value("myFileField.default"))
 
-    try: self.assertEqual("-photo.png", sel.get_value("myFileField.name")[-10:])
-    except AssertionError, e: self.verificationErrors.append(str(e))
+    self.assertTrue(sel.get_value("myFileField.name"))
 
     sel.type("form-myFileField", os.path.abspath("testdata/photo.jpg"))
     sel.type("form-required", 'foo')
-    sel.click("form-action-submit")
+    sel.click("form-action")
     sel.wait_for_page_to_load("30000")
 
-    try: self.assertEqual(sel.get_attribute("//div[@id='actual']/img@src")[-9:],"photo.jpg")
-    except AssertionError, e: self.verificationErrors.append(str(e))
-    try: self.assertEqual(sel.get_attribute("//div[@id='resized']/img@src")[-22:-13],"photo.jpg")
-    except AssertionError, e: self.verificationErrors.append(str(e))
+    self.assertEqual(sel.get_attribute("//div[@id='actual']/img@src")[-9:],"photo.jpg")
+    self.assertEqual(sel.get_attribute("//div[@id='resized']/img@src")[-22:-13],"photo.jpg")
 
 
     filesrc = sel.get_attribute("//div[@id='actual']/img@src")
@@ -803,11 +808,11 @@ def functest_RequiredStringAndFile(self):
     stdout = subprocess.Popen([IDENTIFY, '/tmp/testish-functest'], stdout=subprocess.PIPE).communicate()[0]
     assert '/tmp/testish-functest JPEG 300x300' in stdout
     
-    actualfilepath = 'store_%s-100x100'%filesrc
+    actualfilepath = '_%s-100x100'%filesrc
     actualfilepath = safefilename.encode(actualfilepath)
     assert os.path.exists('cache/%s'%actualfilepath)
     fs = filestore.FileSystemHeaderedFilestore(root_dir='cache')
-    headers, fp = fs.get('store_%s-100x100'%filesrc)
+    headers, fp = fs.get('_%s-100x100'%filesrc)
     f = open('/tmp/testish-functest','wb')
     shutil.copyfileobj(fp, f)
     f.close()
@@ -918,28 +923,23 @@ def functest_SelectWithOtherChoice(self):
     sel = self.selenium
     sel.open("/SelectWithOtherChoice")
 
-    try: self.assertEqual("", sel.get_value("form-mySelect"))
-    except AssertionError, e: self.verificationErrors.append(str(e))
+    self.assertEqual("", sel.get_value("form-mySelect"))
     sel.select("form-mySelect", "label=a")
-    sel.click("form-action-submit")
+    sel.click("form-action")
     sel.wait_for_page_to_load("30000")
-    try: self.failUnless(sel.is_text_present("{'mySelect': 1}"))
-    except AssertionError, e: self.verificationErrors.append(str(e))
+    self.failUnless(sel.is_text_present("{'mySelect': 1}"))
     sel.select("form-mySelect", "label=Other ...")
     sel.type("form-mySelect-other", "4")
-    sel.click("form-action-submit")
+    sel.click("form-action")
     sel.wait_for_page_to_load("30000")
-    try: self.failUnless(sel.is_text_present("{'mySelect': 4}"))
-    except AssertionError, e: self.verificationErrors.append(str(e))
+    self.failUnless(sel.is_text_present("{'mySelect': 4}"))
     sel.select("form-mySelect", "label=Other ...")
     sel.type("form-mySelect-other", "d")
-    sel.click("form-action-submit")
+    sel.click("form-action")
     sel.wait_for_page_to_load("30000")
-    try: self.failUnless(sel.is_text_present("Not a valid integer"))
-    except AssertionError, e: self.verificationErrors.append(str(e))
+    self.failUnless(sel.is_text_present("Not a valid integer"))
 
-    try: self.assertEqual("...", sel.get_value("form-mySelect"))
-    except AssertionError, e: self.verificationErrors.append(str(e))
+    self.assertEqual("...", sel.get_value("form-mySelect"))
 
     return
 
@@ -1005,20 +1005,17 @@ def functest_RadioChoiceNoneOption(self):
     sel = self.selenium
     sel.open("/RadioChoiceNoneOption")
 
-    sel.click("form-action-submit")
+    sel.click("form-action")
     sel.wait_for_page_to_load("30000")
-    try: self.failUnless(sel.is_text_present("{'myRadio': None}"))
-    except AssertionError, e: self.verificationErrors.append(str(e))
+    self.failUnless(sel.is_text_present("{'myRadio': None}"))
     sel.click("form-myRadio-noneoption")
-    sel.click("form-action-submit")
+    sel.click("form-action")
     sel.wait_for_page_to_load("30000")
-    try: self.failUnless(sel.is_text_present("{'myRadio': None}"))
-    except AssertionError, e: self.verificationErrors.append(str(e))
+    self.failUnless(sel.is_text_present("{'myRadio': None}"))
     sel.click("form-myRadio-0")
-    sel.click("form-action-submit")
+    sel.click("form-action")
     sel.wait_for_page_to_load("30000")
-    try: self.failUnless(sel.is_text_present("{'myRadio': 1}"))
-    except AssertionError, e: self.verificationErrors.append(str(e))
+    self.failUnless(sel.is_text_present("{'myRadio': 1}"))
 
     return
 
@@ -1238,6 +1235,22 @@ def form_SimpleStructure(request):
     form = formish.Form(schema, 'form')
     return form
   
+def form_StructureWithReadonly(request):
+    """
+    A simple structure
+    """
+    structure = schemaish.Structure()
+    structure.add( 'a', schemaish.String() )
+    structure.add( 'b', schemaish.Integer() )
+
+    schema = schemaish.Structure()
+    schema.add( 'myStruct', structure )
+
+    form = formish.Form(schema, 'form')
+    form['myStruct.b'].widget = formish.Input(readonly=True)
+    form['myStruct.b'].default = 7
+    return form
+
 def form_UploadStructure(request):
     """
     A structure with a file upload
@@ -1261,6 +1274,16 @@ def form_UploadStructure(request):
 #   Sequences
 #
 
+def form_SequenceOfStringsWithSequenceWidgetOptions(request):
+    """
+    A sequence with some defaults
+    """
+    schema = schemaish.Structure()
+    schema.add( 'myList', schemaish.Sequence( schemaish.String() ))
+
+    form = formish.Form(schema, 'form')
+    form['myList'].widget = formish.SequenceDefault(min_start_fields=1,min_empty_start_fields=0, batch_add_count=5)
+    return form
 
 def form_SequenceOfSimpleStructures(request):
     """
@@ -1274,6 +1297,7 @@ def form_SequenceOfSimpleStructures(request):
     schema.add( 'myList', schemaish.Sequence( substructure ))
 
     form = formish.Form(schema, 'form')
+    form['myList'].widget = formish.SequenceDefault()
     return form
   
 def form_SequenceOfUploadStructures(request):
@@ -1297,17 +1321,14 @@ def functest_SequenceOfUploadStructures(self):
 
     sel.open("/SequenceOfUploadStructures")
     sel.wait_for_page_to_load("30000")
-    sel.mouse_down("link=Add")
 
     sel.type("form-myList-0-a", os.path.abspath("testdata/test.txt"))
     sel.type("form-myList-0-b", "13")
-    sel.click("form-action-submit")
+    sel.click("form-action")
     sel.wait_for_page_to_load("30000")
 
-    try: self.failUnless(sel.is_text_present("{'myList': [{'a': <schemaish.type.File"))
-    except AssertionError, e: self.verificationErrors.append(str(e))
-    try: self.failUnless(sel.is_text_present("'b': 13}]}"))
-    except AssertionError, e: self.verificationErrors.append(str(e))
+    self.failUnless(sel.is_text_present("{'myList': [{'a': <schemaish.type.File"))
+    self.failUnless(sel.is_text_present("'b': 13}]}"))
 
     return
 
@@ -1334,18 +1355,14 @@ def functest_SequenceOfStructuresWithSelects(self):
     sel = self.selenium
     sel.open("/SequenceOfStructuresWithSelects")
 
-    try: self.assertEqual("foo", sel.get_value("form-myList-0-a"))
-    except AssertionError, e: self.verificationErrors.append(str(e))
+    self.assertEqual("foo", sel.get_value("form-myList-0-a"))
 
-    try: self.assertEqual("b", sel.get_value("form-myList-0-b"))
-    except AssertionError, e: self.verificationErrors.append(str(e))
+    self.assertEqual("b", sel.get_value("form-myList-0-b"))
 
     sel.click_at("css=#form-myList--field > a", "")
-    try: self.assertEqual("", sel.get_value("form-myList-1-a"))
-    except AssertionError, e: self.verificationErrors.append(str(e))
+    self.assertEqual("", sel.get_value("form-myList-1-a"))
 
-    try: self.assertEqual("", sel.get_value("form-myList-1-b"))
-    except AssertionError, e: self.verificationErrors.append(str(e))
+    self.assertEqual("", sel.get_value("form-myList-1-b"))
 
     return
 
@@ -1370,8 +1387,7 @@ def functest_SequenceOfDateParts(self):
     sel.open("/SequenceOfDateParts")
 
     sel.click_at("css=#form-birthdays--field > a", "")
-    try: self.assertEqual("", sel.get_value("form-birthdays-0-date"))
-    except AssertionError, e: self.verificationErrors.append(str(e))
+    self.assertEqual("", sel.get_value("form-birthdays-0-date"))
 
     return
 
@@ -1385,6 +1401,16 @@ def form_SequenceOfSequencesAsTextArea(request):
     form['table'].widget = formish.TextArea()
     return form
     
+def form_SequenceAsInputWithDefaultAndDelimiter(request):
+    """
+    A simple text area but representing a csv style data structure
+    """
+    schema = schemaish.Structure()
+    schema.add('a', schemaish.Tuple( (schemaish.String(), schemaish.Integer()) ))
+    form = formish.Form(schema)
+    form['a'].widget = formish.Input(converter_options={'delimiter':':'})
+    form.defaults = {'a': ('a',3)}
+    return form
 
 def form_SequenceOfStructures(request):
     """
@@ -1564,3 +1590,53 @@ ${form.footer()|n}
 
     """
 
+def form_CustomisedFormLayoutWithSequence(request):
+    """
+    A custom form
+    """
+    schema = schemaish.Structure()
+    schema.add( 'firstName', schemaish.String())
+    schema.add( 'surname', schemaish.String(description='THIS MUST BE YOUR SURNAME') )
+    email = schemaish.Structure()
+    email.add('type',schemaish.String())
+    email.add('email',schemaish.String())
+    schema.add( 'emails', schemaish.Sequence(email))
+
+    form = formish.Form(schema, 'form')
+
+    form['surname'].widget = formish.Input(css_class="surnamewidget")
+    
+    form.defaults = {'firstName': 'Tim', 'surname': 'Parkin','emails': [{'type':'home','email':'a@b.com'},{'type':'work','email':'c@d.com'}]}
+
+    return form
+
+
+def template_CustomisedFormLayoutWithSequence(request):
+    """
+${form.header()|n}
+${form.metadata()|n}
+
+${form['firstName']()|n}
+
+<div id="${form['surname'].cssname}--field" class="${form['surname'].classes}">
+  <strong>${form['surname'].description}</strong>
+  <em>${form['surname'].error}</em>
+  ${form['surname'].widget()|n}
+  <br />
+  <table>
+  %for field in form['emails'].fields:
+  <tr>
+    <td>${field['email'].title|n}</td>
+    <td> ${field['email'].widget()|n}</td>
+    <td>${field['type'].title|n}</td>
+    <td>${field['type'].widget()|n}</td>
+  </tr>
+  %endfor
+  </table>
+</div>
+
+${form.actions()|n}
+${form.footer()|n}
+
+    """
+
