@@ -1,7 +1,7 @@
 import formish
 import unittest
 import schemaish
-from dottedish import dotted
+from dottedish.api import dotted, unflatten
 from formish.forms import validation
 import copy
 from webob import MultiDict
@@ -23,6 +23,13 @@ class Request(object):
         getattr(self, method.upper())['__formish_form__'] = form_name
         # Set the method
         self.method = method
+
+    def get_post(self):
+        def container_factory(parent_key, item_key):
+            if item_key.isdigit():
+                return []
+            return {}
+        return unflatten(self.POST.dict_of_lists().iteritems(), container_factory=container_factory)
         
             
 class TestFormBuilding(unittest.TestCase):
@@ -41,7 +48,26 @@ class TestFormBuilding(unittest.TestCase):
         self.assertEqual(form.name,name)
         # this is really empty
         assert list(form.fields) == []
-        
+
+    def test_readonly_field(self):
+        """Test a form that has no a read only field"""
+        schema_readonly = schemaish.Structure([("a", schemaish.String()), ("b", schemaish.String())])
+        name = "Flat Form"
+        request =  Request(name)
+        form = formish.Form(schema_readonly, name)
+        form['a'].widget=formish.Input(readonly=True)
+        request = Request(name, {'b': 'bar'})
+        actualdata = form.validate(request)
+        expecteddata = {'a': None,'b': 'bar'}
+        print actualdata, expecteddata
+        assert  actualdata == expecteddata
+        form.defaults = {'a': 'foo'}
+        actualdata = form.validate(request)
+        expecteddata = {'a': 'foo','b': 'bar'}
+        print actualdata, expecteddata
+        assert  actualdata == expecteddata
+        print form()
+
 
     def test_flat_form(self):
         """Test a form that has no nested sections """
@@ -72,33 +98,6 @@ class TestFormBuilding(unittest.TestCase):
         assert len(list(form.fields)) == 2
 
 
-    def test_data_and_request_conversion(self):
-        """
-        Test convert request to data and convert data to request
-        """
-        schema_nested = schemaish.Structure([
-            ("one", schemaish.Structure([
-                ("a", schemaish.String()),
-                ("b", schemaish.String()),
-                ("c", schemaish.Structure([("x", schemaish.String()),("y", schemaish.String())])),
-                ])
-             ),
-            ])
-        r = {'one.a':'','one.b': '','one.c.x': '','one.c.y': ''}
-        reqr = {'one.a':None,'one.b': None,'one.c.x': None,'one.c.y': None}
-        reqrdata = {'one.a':[''],'one.b': [''],'one.c.x': [''],'one.c.y': ['']}
-        data = {'one.a': '', 'one.b': '', 'one.c.x': '', 'one.c.y': ''}
-        
-        name = "Nested Form Two"
-        request =  Request(name, r)
-        form = formish.Form(schema_nested, name)
-        # request to data
-        rdtd = validation.from_request_data(form.structure, dotted(copy.deepcopy(request.POST)))
-        assert rdtd == dotted(reqr)
-        # data to request
-        dtrd = validation.to_request_data(form.structure, dotted(data))
-        assert dtrd == reqrdata
-
 
     def test_nested_form_validation_errors(self):
         schema_nested = schemaish.Structure([
@@ -155,9 +154,9 @@ class TestFormBuilding(unittest.TestCase):
         # Does the form produce an int and a string
         self.assertEquals(form.validate(request), {'a': 3, 'b': '4'})
         # Does the convert request to data work
-        self.assertEqual( validation.from_request_data(form.structure, dotted(request.POST)) , {'a': 3, 'b': '4'})
+        self.assertEqual( form.widget.from_request_data(form, request.POST) , {'a': 3, 'b': '4'})
         # Does the convert data to request work
-        self.assert_( validation.to_request_data(form.structure, dotted( {'a': 3, 'b': '4'} )) == reqr)
+        self.assert_( form.widget.to_request_data(form, {'a': 3, 'b': '4'} ) == reqr)
 
     def test_failure_and_success_callables(self):
 
@@ -196,43 +195,47 @@ class TestFormBuilding(unittest.TestCase):
         # Check the data is converted correctly
         self.assertEquals(form.validate(request), {'a': d, 'b': '4'})
         # Check req to data
-        self.assertEqual( validation.from_request_data(form.structure, dotted(request.POST)) , dotted({'a': d, 'b': '4'}))
+        print 'repr post',repr(request.get_post())
+        self.assertEqual( form.widget.from_request_data(form, request.get_post()) , {'a': d, 'b': '4'})
         # Check data to req
-        self.assert_( validation.to_request_data(form.structure, dotted( {'a': d, 'b': '4'} )) == dotted({'a': {'month': [3], 'day': [1], 'year': [1966]}, 'b': ['4']}))
+        self.assert_( form.widget.to_request_data(form, {'a': d, 'b': '4'}) == {'a': {'month': [3], 'day': [1], 'year': [1966]}, 'b': ['4']})
 
     def test_form_retains_request_data(self):
-        form = formish.Form(schemaish.Structure([("field", schemaish.String())]))
+        form = formish.Form(schemaish.Structure([("field", schemaish.String())]),'form')
         assert 'name="field" value=""' in form()
         data = form.validate(Request('form', {'field': 'value'}))
+        print 'data',data
         assert data == {'field': 'value'}
+        print 'form.request_data[field] =',form.request_data['field']
         assert form.request_data['field'] == ['value']
         assert 'name="field" value="value"' in form()
 
     def test_form_accepts_request_data(self):
         form = formish.Form(schemaish.Structure([("field", schemaish.String())]))
         form.request_data = {'field': ['value']}
-        assert form.request_data == {'field': ['value']}
+        print 'FRD',form._request_data
+        assert form._request_data == {'field': ['value']}
 
     def test_form_with_defaults_accepts_request_data(self):
         form = formish.Form(schemaish.Structure([("field", schemaish.String())]))
         form.defaults = {'field': 'default value'}
         assert 'name="field" value="default value"' in form()
         form.request_data = {'field': ['value']}
-        assert form.request_data == {'field': ['value']}
+        assert form._request_data == {'field': ['value']}
         assert 'name="field" value="value"' in form()
 
     def test_form_defaults_clears_request_data(self):
         form = formish.Form(schemaish.Structure([("field", schemaish.String())]))
         form.request_data = {'field': ['value']}
         form.defaults = {'field': 'default value'}
-        assert form.defaults == {'field': 'default value'}
-        assert form.request_data == {'field': ['default value']}
+        assert form._defaults == {'field': 'default value'}
+        assert form._request_data == None
         assert 'name="field" value="default value"' in form()
 
     def test_method(self):
         schema = schemaish.Structure([('string', schemaish.String())])
         # Default should be POST
-        self.assertTrue('method="post"' in formish.Form(schema)().lower())
+        self.assertTrue('method="post"' in formish.Form(schema,'form')().lower())
         # (Crudely) check that an explicit method is rendered correctly by the
         # templates.
         for method in ['POST', 'GET', 'get', 'Get']:
@@ -244,9 +247,25 @@ class TestFormBuilding(unittest.TestCase):
         # Check that default (POST) and non-default (e.g. GET) forms validate.
         for method, request in [('post', Request(POST={'string': 'abc'})),
                                 ('get', Request(GET={'string': 'abc'}, method='GET'))]:
-            data = formish.Form(schema, method=method).validate(request)
+            data = formish.Form(schema,'form', method=method).validate(request)
             self.assertTrue(data == {'string': 'abc'})
 
+    def test_simple_validation(self):
+        schema_flat = schemaish.Structure([("a", schemaish.Integer())])
+        name = "Integer Form"
+        form = formish.Form(schema_flat, name)
+        r = {'a': '3'}
+        request = Request(name, r)
+        R = copy.deepcopy(r)
+
+        reqr = {'a': ['3']}
+        # Does the form produce an int and a string
+        self.assertEquals(form.validate(request), {'a': 3})
+        # Does the convert request to data work
+        # self.assertEqual( validation.from_request_data(form.structure, dotted(request.POST)) , {'a': 3, 'b': '4'})
+        # Does the convert data to request work
+        # self.assert_( validation.to_request_data(form.structure, dotted( {'a': 3, 'b': '4'} )) == reqr)
+
 
 class TestActions(unittest.TestCase):
 
@@ -272,9 +291,9 @@ class TestActions(unittest.TestCase):
         def callback2(*a, **k):
             return 'two'
         schema = schemaish.Structure([('string', schemaish.String())])
-        form = formish.Form(schema, method=method)
-        form.add_action(callback1, 'one')
-        form.add_action(callback2, 'two')
+        form = formish.Form(schema, method=method, add_default_action=False)
+        form.add_action('one', 'One', callback1)
+        form.add_action('two', 'Two', callback2)
         return form
 
 
@@ -291,7 +310,7 @@ class TestBugs(unittest.TestCase):
     def test_date_conversion(self):
         from datetime import date, datetime
         schema = schemaish.Structure([('date', schemaish.Date())])
-        form = formish.Form(schema)
+        form = formish.Form(schema,name='form')
         form['date'].widget = formish.SelectChoice([(date(1970,1,1),'a'),
                                                        (date(1980,1,1),'b'),
                                                        (datetime(1990,1,1),'c')])
