@@ -21,6 +21,7 @@ class TestHTML(unittest.TestCase):
         
     def test_error_html(self):
         r = webob.Request.blank('http://localhost/', environ={'REQUEST_METHOD': 'POST'})
+        r.POST['__formish_form__'] = 'form'
         r.POST['one'] = ''
         schema = schemaish.Structure([('one', schemaish.String(validator=v.Required()))])
         f = formish.Form(schema,name="form")
@@ -29,7 +30,7 @@ class TestHTML(unittest.TestCase):
         except fv.FormError, e:
             assert str(f.errors['one']) == 'is required'
         soup = BeautifulSoup(f())
-        assert soup.find(id='form-one-field').find("span", "error").string == 'is required' , "test that the form error is being created"
+        assert soup.find(id='form-one--field').find("span", "error").string == 'is required' , "test that the form error is being created"
         
     def test_complex_form(self):
         
@@ -38,7 +39,7 @@ class TestHTML(unittest.TestCase):
                          ('c', schemaish.Sequence(schemaish.String())), ("d", schemaish.String()), \
                          ("e", schemaish.Integer(validator=v.Required())), ("f", schemaish.String(validator=v.Required())) ])
         schema = schemaish.Structure([("one", one), ("two", two)])
-        f = formish.Form(schema,name="form")
+        f = formish.Form(schema,name="form",add_default_action=False)
 
         f['one.b'].widget = formish.TextArea()
         f['two.a'].widget = formish.SelectChoice([('opt1',"Options 1"),('opt2',"Option 2")], none_option=('-select option-',None))
@@ -47,12 +48,15 @@ class TestHTML(unittest.TestCase):
         f['two.d'].widget = formish.RadioChoice([('opt1',"Options 1"),('opt2',"Option 2")])
         f['two.f'].widget = formish.CheckedPassword()
 
-        f.add_action(lambda x: x, 'submit', label="Submit Me")
+        f.add_action('submit', "Submit Me")
         f.defaults = {'one': {'a' : 'ooteenee','c':['3','4','5']}, 'two': {'a': 'opt1','b': date(1966,1,3),'c':['opt2'],'d':'opt2'} } 
-        soup = BeautifulSoup(f())
+        print '*****'
+        print f()
+        print '*****'
+        #soup = BeautifulSoup(f())
         ## Latch the results for acceptance tests
         #open('formish/tests/expectations/test_complex_form.html','w').write(html)
-        expectedSoup = BeautifulSoup( open('formish/tests/unittests/expectations/test_complex_form.html').read())
+        #expectedSoup = BeautifulSoup( open('formish/tests/unittests/expectations/test_complex_form.html').read())
         #assert soup == expectedSoup
 
 
@@ -61,27 +65,28 @@ class TestHTML(unittest.TestCase):
         schema = schemaish.Structure([("one", schemaish.Integer(validator=v.All(
                        v.Required(), v.Integer(), v.Range(min=18), v.Range(min=20), 
                    ))),])
-        f = formish.Form(schema,name="form")
+        f = formish.Form(schema,name="form",add_default_action=False)
 
-        f.add_action(lambda x: x, 'submit', label="Submit Me")
+        f.add_action('submit', "Submit Me")
         r = webob.Request.blank('http://localhost/', environ={'REQUEST_METHOD': 'POST'})
+        r.POST['__formish_form__'] = 'form'
         r.POST['one'] = '9'
         try:
             data = f.validate(r)
         except fv.FormError, e:
-            print '---',f.errors['one']
             assert str(f.errors['one']) == 'must be greater than 18; must be greater than 20'
             assert str(f['one'].field.errors.exceptions[0]) == 'must be greater than 18'
 
-    def test_complex_error_all_required(self):
+    def test_complex_error_all_required_1(self):
         
         schema = schemaish.Structure([("one", schemaish.Integer(validator=v.All(
                        v.Required(), v.Integer(), v.Range(min=18), v.Range(min=20), 
                    ))),])
-        f = formish.Form(schema,name="form")
+        f = formish.Form(schema,name="form",add_default_action=False)
 
-        f.add_action(lambda x: x, 'submit', label="Submit Me")
+        f.add_action('submit', "Submit Me")
         r = webob.Request.blank('http://localhost/', environ={'REQUEST_METHOD': 'POST'})
+        r.POST['__formish_form__'] = 'form'
         r.POST['one'] = ''
         try:
             data = f.validate(r)
@@ -90,15 +95,16 @@ class TestHTML(unittest.TestCase):
             assert str(f.errors['one']) == 'is required; must be greater than 18; must be greater than 20'
             assert str(f['one'].field.errors.exceptions[0]) == 'is required'
 
-    def test_complex_error_all_required(self):
+    def test_complex_error_all_required_2(self):
         
         schema = schemaish.Structure([("one", schemaish.Integer(validator=v.All(
                        v.Required(), v.Integer(), v.Any(v.Range(min=18), v.Range(min=20)), 
                    ))),])
-        f = formish.Form(schema,name="form")
+        f = formish.Form(schema,name="form", add_default_action=False)
 
-        f.add_action(lambda x: x, 'submit', label="Submit Me")
+        f.add_action('submit', "Submit Me")
         r = webob.Request.blank('http://localhost/', environ={'REQUEST_METHOD': 'POST'})
+        r.POST['__formish_form__'] = 'form'
         r.POST['one'] = ''
         try:
             data = f.validate(r)
