@@ -2,19 +2,46 @@
 Commonly needed form widgets.
 """
 
-__all__ = ['Input', 'Password', 'CheckedPassword', 'Hidden', 'TextArea',
+__all__ = ['Input', 'Password', 'CheckedPassword', 'CheckedInput', 'Hidden', 'TextArea',
         'Checkbox', 'DateParts', 'FileUpload', 'SelectChoice','SelectWithOtherChoice','RadioChoice',
         'CheckboxMultiChoice', 'SequenceDefault','CheckboxMultiChoiceTree', 'Grid']
 
 from convertish.convert import string_converter, \
         datetuple_converter,ConvertError
 from schemaish.type import File as SchemaFile
-from dottedish import get_dict_from_dotted_dict
 import uuid
 
+from formish import util
+from formish.filestore import CachedTempFilestore
+from validatish import Invalid
+
 
 UNSET = object()
 
+def recursive_convert_sequences(data):
+    """
+    recursively applies ``convert_sequences``
+    """
+    if not hasattr(data,'keys'):
+        return data
+    if len(data.keys()) == 0:
+        return data
+    try:
+        int(data.keys()[0])
+    except ValueError:
+        tmp = {}
+        for key, value in data.items():
+            tmp[key] = recursive_convert_sequences(value)
+        return tmp
+    intkeys = []
+    for key in data.keys():
+        intkeys.append(int(key))
+    intkeys.sort()
+    out = []
+    for key in intkeys:
+        out.append(recursive_convert_sequences(data[str(key)]))
+    return out
+
 
 class Widget(object):
     """
@@ -23,17 +50,18 @@ class Widget(object):
     
     type = None
     template = None
+    default_value = ['']
     
     def __init__(self, **k):
         self.css_class = k.get('css_class', None)
-        self.converttostring = True
         self.empty = k.get('empty', None)
+        self.readonly = k.get('readonly',False)
         self.converter_options = k.get('converter_options', {})
         if not self.converter_options.has_key('delimiter'):
             self.converter_options['delimiter'] = ','
     
 
-    def to_request_data(self, schema_type, data):
+    def to_request_data(self, field, data):
         """
         Before the widget is rendered, the data is converted to a string
         format.If the data is None then we return an empty string. The sequence
@@ -41,19 +69,19 @@ class Widget(object):
         """
         if data is None:
             return ['']
-        string_data = string_converter(schema_type).from_type(data)
+        string_data = string_converter(field.attr).from_type(data, converter_options=self.converter_options)
         return [string_data]
 
 
-    def pre_parse_incoming_request_data(self, schema_type, request_data, full_request_data):
+    def pre_parse_incoming_request_data(self, field, request_data):
         """
         Prior to convert being run, we have a chance to munge the data. This is
         only used by file upload at the moment
         """
-        return request_data
+        return request_data or self.default_value
 
 
-    def from_request_data(self, schema_type, request_data):
+    def from_request_data(self, field, request_data):
         """
         after the form has been submitted, the request data is converted into
         to the schema type.
@@ -61,12 +89,10 @@ class Widget(object):
         string_data = request_data[0]
         if string_data == '':
             return self.empty
-        return string_converter(schema_type).to_type(string_data)
+        return string_converter(field.attr).to_type(string_data, converter_options=self.converter_options)
 
 
 
-    def __call__(self, field):
-        return field.form.renderer('/formish/widgets/%s.html'%self.template, {'f':field})
 
     def __repr__(self):
         attributes = []
@@ -95,7 +121,7 @@ class Input(Widget):
         if not self.converter_options.has_key('delimiter'):
             self.converter_options['delimiter'] = ','
 
-    def from_request_data(self, schema_type, request_data):
+    def from_request_data(self, field, request_data):
         """
         Default to stripping whitespace
         """
@@ -104,7 +130,7 @@ class Input(Widget):
             string_data = string_data.strip()
         if string_data == '':
             return self.empty
-        return string_converter(schema_type).to_type(string_data)
+        return string_converter(field.attr).to_type(string_data, converter_options=self.converter_options)
 
     def __repr__(self):
         attributes = []
@@ -136,24 +162,25 @@ class CheckedPassword(Input):
 
     type = 'CheckedPassword'
     template = 'field.CheckedPassword'
+    default_value = {'password': [''], 'confirm': ['']}
 
     def __init__(self, **k):
         self.strip = k.pop('strip', True)
         self.css_class = k.pop('css_class', None)
         Input.__init__(self, **k)
         if not self.converter_options.has_key('delimiter'):
-            self.converter_options['delimiter'] = ','
+            self.empty.converter_options['delimiter'] = ','
             
-    def to_request_data(self, schema_type, data):
+    def to_request_data(self, field, data):
         """
         Extract both the password and confirm fields
         """
-        string_data = string_converter(schema_type).from_type(data)
+        string_data = string_converter(field.attr).from_type(data)
         if string_data is None:
             return {'password': [''], 'confirm': ['']}
         return {'password': [string_data], 'confirm': [string_data]}
     
-    def from_request_data(self, schema_type, request_data):
+    def from_request_data(self, field, request_data):
         """
         Check the password and confirm match (when stripped)
         """
@@ -166,7 +193,61 @@ class CheckedPassword(Input):
             raise ConvertError('Password did not match')
         if password == '':
             return self.empty
-        return string_converter(schema_type).to_type(password)
+        return string_converter(field.attr).to_type(password)
+
+    def __repr__(self):
+        attributes = []
+        if self.strip is False:
+            attributes.append('strip=%r'%self.strip)
+        if self.converter_options != {'delimiter':','}:
+            attributes.append('converter_options=%r'%self.converter_options)
+        if self.css_class:
+            attributes.append('css_class=%r'%self.css_class)
+        if self.empty is not None:
+            attributes.append('empty=%r'%self.empty)
+
+        return 'formish.%s(%s)'%(self.__class__.__name__, ', '.join(attributes))
+
+
+class CheckedInput(Input):
+    """
+    Checked Input ensures that the input has been entered twice
+    """
+
+    type = 'CheckedInput'
+    template = 'field.CheckedInput'
+    default_value = {'input': [''], 'confirm': ['']}
+
+    def __init__(self, **k):
+        self.strip = k.pop('strip', True)
+        self.css_class = k.pop('css_class', None)
+        Input.__init__(self, **k)
+        if not self.converter_options.has_key('delimiter'):
+            self.converter_options['delimiter'] = ','
+            
+    def to_request_data(self, field, data):
+        """
+        Extract both the input and confirm fields
+        """
+        string_data = string_converter(field.attr).from_type(data)
+        if string_data is None:
+            return {'input': [''], 'confirm': ['']}
+        return {'input': [string_data], 'confirm': [string_data]}
+    
+    def from_request_data(self, field, request_data):
+        """
+        Check the input and confirm match (when stripped)
+        """
+        input = request_data['input'][0]
+        confirm = request_data['confirm'][0]
+        if self.strip is True:
+            input = input.strip()
+            confirm = confirm.strip()
+        if input != confirm:
+            raise ConvertError('Fields did not match')
+        if input == '':
+            return self.empty
+        return string_converter(field.attr).to_type(input)
 
     def __repr__(self):
         attributes = []
@@ -190,6 +271,31 @@ class Hidden(Input):
     template = 'field.Hidden'
 
 
+def default_empty_checker(v):
+    try:
+        _default_empty_checker(v)
+    except ValueError:
+        return False
+    return True
+
+
+def _default_empty_checker(v):
+    if v == None:
+        return
+    if isinstance(v, basestring):
+        if v != '':
+            raise ValueError
+        return
+    if hasattr(v, 'values'):
+        for i in v.values():
+            _default_empty_checker(i) 
+        return
+    else:
+        try:
+            for i in v: 
+                _default_empty_checker(i)
+        except TypeError:
+            raise ValueError
 
 class SequenceDefault(Widget):
     """
@@ -206,18 +312,83 @@ class SequenceDefault(Widget):
 
     def __init__(self, **k):
         Widget.__init__(self, **k)
-        self.max = k.get('max')
-        self.min = k.get('min')
+        self.empty_checker = k.get('empty_checker',default_empty_checker)
+        self.min_start_fields = k.get('min_start_fields',1)
+        self.min_empty_start_fields = k.get('min_empty_start_fields',0)
+        self.batch_add_count = k.get('batch_add_count',1)
         self.addremove = k.get('addremove', True)
         self.sortable = k.get('sortable', True)
-        self.converttostring = False
+        # you can specify strip_empty but if not then we only strip if you can add and remove items
+        self.strip_empty = k.get('strip_empty',self.addremove)
 
-    def to_request_data(self, schema_type, data):
+    def to_request_data(self, field, data):
         """
         Short circuits the usual to_request_data
         """
+        request_data = {}
+        if data is None:
+            data = {}
+        for f in field.fields:
+            try:
+                try:
+                    d = data[int(f.nodename)]
+                except (KeyError, IndexError):
+                    d = None
+                request_data[f.nodename] = f.widget.to_request_data(f, d)
+            except Invalid, e:
+                f.errors[f.name] = e
+                raise
+        return request_data
+
+    def pre_parse_incoming_request_data(self, field, request_data):
+        """
+        Some widgets (at the moment only files) need to have their data massaged in
+        order to make sure that data->request and request->data is symmetric
+
+        This pre parsing is a null operation for most widgets
+        """
+        data = {}
+
+        for f in field.fields:
+            try:
+                r = request_data[int(f.nodename)]
+            except TypeError, KeyError:
+                r = None
+            d = f.widget.pre_parse_incoming_request_data(f, r)
+            data[f.nodename] = d
+
         return data
 
+    def from_request_data(self, field, request_data, skip_read_only_default=False):
+        data = []
+        if request_data is None:
+            request_data = {}
+        for f in field.fields:
+            try:
+                if f.widget.readonly is not True:
+                    data.append( f.widget.from_request_data(f, request_data.get(f.nodename)) )
+                else:
+                    if skip_read_only_default is False:
+                        data.append( f.defaults )
+            except ConvertError, e:
+                f.errors = e.message
+
+        # Trim empty fields from the end of the list
+        if self.strip_empty:
+            n = None
+            for n in xrange(len(data),0,-1):
+                if not self.empty_checker(data[n-1]):
+                    break
+            else:
+                # they are all empty
+                n = 0
+            if n is not None:
+                return data[:n]
+        return data
+
+
+        
+
     def __repr__(self):
         attributes = []
         if self.converter_options != {'delimiter':','}:
@@ -226,17 +397,84 @@ class SequenceDefault(Widget):
             attributes.append('css_class=%r'%self.css_class)
         if self.empty is not None:
             attributes.append('empty=%r'%self.empty)
-        if self.min:
-            attributes.append('min=%r'%self.min)
-        if self.max:
-            attributes.append('max=%r'%self.max)
+        if self.batch_add_count:
+            attributes.append('batch_add_count=%r'%self.batch_add_count)
+        if self.min_start_fields:
+            attributes.append('min_start_fields=%r'%self.min_start_fields)
+        if self.min_empty_start_fields:
+            attributes.append('min_empty_start_fields=%r'%self.min_empty_start_fields)
         if self.addremove:
             attributes.append('addremove=%r'%self.addremove)
         if self.sortable:
             attributes.append('sortable=%r'%self.sortable)
 
         return 'formish.%s(%s)'%(self.__class__.__name__, ', '.join(attributes))
+
+
+class StructureDefault(Widget):
+    """
+    Sequence handling widget - used by default for schema sequences
+
+    :arg min: minimum number of sequence items to show
+    :arg max: maximum number of sequence items to show
+    :arg addremove: boolean whether to show the addremove buttons (jquery
+        activated)
+    """
+
+    type = 'StructureDefault'
+    template = 'structure.-'
+
+    def to_request_data(self, field, data):
+        """
+        Short circuits the usual to_request_data
+        """
+        request_data = {}
+        if data is None:
+            data = {}
+        for f in field.fields:
+            try:
+                request_data[f.nodename] = f.widget.to_request_data(f, data.get(f.nodename))
+            except Invalid, e:
+                f.errors[f.name] = e.message
+                raise
+        return request_data
+
+    def pre_parse_incoming_request_data(self, field, request_data):
+        """
+        Some widgets (at the moment only files) need to have their data massaged in
+        order to make sure that data->request and request->data is symmetric
+
+        This pre parsing is a null operation for most widgets
+        """
+        data = {}
+
+        for f in field.fields:
+            if request_data is None:
+                r = None
+            else:
+                r = request_data.get(f.nodename, None)
+            d = f.widget.pre_parse_incoming_request_data(f, r)
+            data[f.nodename] = d
+        return data
+
+    def from_request_data(self, field, request_data, skip_read_only_defaults=False):
+        data = {}
+        if request_data is None:
+            request_data = {}
+        for f in field.fields:
+            try:
+                if f.widget.readonly is not True:
+                    data[f.nodename] = f.widget.from_request_data(f, request_data.get(f.nodename))
+                else:
+                    if skip_read_only_defaults is False:
+                        data[f.nodename] = f.defaults
+            except ConvertError, e:
+                f.errors = e.message
         
+        return data
+
+    
+
 class TextArea(Input):
     """
     Textarea input field
@@ -256,18 +494,17 @@ class TextArea(Input):
         if not self.converter_options.has_key('delimiter'):
             self.converter_options['delimiter'] = '\n'
     
-    def to_request_data(self, schema_type, data):
+    def to_request_data(self, field, data):
         """
         We're using the converter options to allow processing sequence data
         using the csv module
         """
-        string_data = string_converter(schema_type).from_type(data, \
-            converter_options=self.converter_options)
+        string_data = string_converter(field.attr).from_type(data, converter_options=self.converter_options)
         if string_data is None:
             return ['']
         return [string_data]
     
-    def from_request_data(self, schema_type, request_data):
+    def from_request_data(self, field, request_data):
         """
         We're using the converter options to allow processing sequence data
         using the csv module
@@ -277,8 +514,7 @@ class TextArea(Input):
             string_data = string_data.strip()
         if string_data == '':
             return self.empty
-        return string_converter(schema_type).to_type(string_data,
-            converter_options=self.converter_options)
+        return string_converter(field.attr).to_type(string_data, converter_options=self.converter_options)
 
     def __repr__(self):
         attributes = []
@@ -293,38 +529,13 @@ class TextArea(Input):
 
         return 'formish.%s(%s)'%(self.__class__.__name__, ', '.join(attributes))
 
-class Grid(Input):
+class Grid(SequenceDefault):
     """
     Grid input field
     """
-
     type = 'Grid'
     template = 'field.Grid'
     
-    def __init__(self, **k):
-        Input.__init__(self, **k)
-        if not self.converter_options.has_key('delimiter'):
-            self.converter_options['delimiter'] = '\n'
-        self.converttostring = False
-    
-    def to_request_data(self, schema_type, data):
-        string_data = string_converter(schema_type).from_type(data, \
-            converter_options=self.converter_options)
-        return [string_data]
-    
-    def from_request_data(self, schema_type, request_data):
-        string_data = request_data[0]
-        return string_converter(schema_type).to_type(string_data,
-            converter_options=self.converter_options)
-
-    def __repr__(self):
-        attributes = []
-        if self.converter_options != {'delimiter':','}:
-            attributes.append('converter_options=%r'%self.converter_options)
-        if self.css_class:
-            attributes.append('css_class=%r'%self.css_class)
-
-        return 'formish.%s(%s)'%(self.__class__.__name__, ', '.join(attributes))
     
 class Checkbox(Widget):
     """
@@ -333,16 +544,30 @@ class Checkbox(Widget):
 
     type = 'Checkbox'
     template = 'field.Checkbox'
+    default_value = [None]
 
-    def from_request_data(self, schema_type, request_data):
+    def __init__(self, checked_value=True, unchecked_value=False, css_class=None):
+        self.checked_value = checked_value
+        self.unchecked_value = unchecked_value
+        Widget.__init__(self, css_class=css_class)
+
+    def to_request_data(self, field, data):
+        string_data = string_converter(field.attr).from_type(data)
+        return [string_data]
+
+    def from_request_data(self, field, request_data):
+        if string_converter(field.attr).to_type(request_data[0]) == self.checked_value:
+            return self.checked_value
+        return self.unchecked_value
+
+    def checked(self, field):
         """
-        If the request data exists, then we treat this as True
+        For each value, convert it and check to see if it matches the input data
         """
-        if len(request_data) == 0:
-            out_string = 'False'
+        if field.value and string_converter(field.attr).to_type(field.value[0]) == self.checked_value:
+            return ' checked="checked"'
         else:
-            out_string = 'True'
-        return string_converter(schema_type).to_type(out_string)
+            return ''
 
     
 class DateParts(Widget):
@@ -352,6 +577,7 @@ class DateParts(Widget):
 
     type = 'DateParts'
     template = 'field.DateParts'
+    default_value = {'year':[''], 'month':[''], 'day': ['']}
     
     def __init__(self, **k):
         self.strip = k.pop('strip', True)
@@ -361,29 +587,29 @@ class DateParts(Widget):
             self.converter_options['delimiter'] = ','
 
         
-    def to_request_data(self, schema_type, data):
+    def to_request_data(self, field, data):
         """
         Convert to date parts
         """
-        dateparts = datetuple_converter(schema_type).from_type(data)
+        dateparts = datetuple_converter(field.attr).from_type(data)
         if dateparts is None:
             return {'year': [''], 'month': [''], 'day': ['']}
         return {'year': [dateparts[0]],
                 'month': [dateparts[1]],
                 'day': [dateparts[2]]}
     
-    def from_request_data(self, schema_type, request_data):
+    def from_request_data(self, field, request_data):
         """
         Pull out the parts and convert
         """
-        year = request_data.get('year', [''])[0].strip()
-        month = request_data.get('month', [''])[0].strip()
-        day = request_data.get('day', [''])[0].strip()
+        year = request_data['year'][0].strip()
+        month = request_data['month'][0].strip()
+        day = request_data['day'][0].strip()
         if year or month or day:
             date_parts = (year, month, day)
         else:
             return self.empty
-        return datetuple_converter(schema_type).to_type(date_parts)
+        return datetuple_converter(field.attr).to_type(date_parts)
 
     def __repr__(self):
         attributes = []
@@ -409,15 +635,12 @@ class FileUpload(Widget):
     type = 'FileUpload'
     template = 'field.FileUpload'
     
-    def __init__(self, filestore, show_file_preview=True, show_download_link=False, \
-                 show_image_thumbnail=False, url_base=None, \
-                 css_class=None, image_thumbnail_default=None, url_ident_factory=None):
+    def __init__(self, filestore=UNSET, show_file_preview=True,
+                 show_download_link=False, show_image_thumbnail=False,
+                 url_base=None, css_class=None, image_thumbnail_default=None,
+                 url_ident_factory=None):
         """
-        :arg filestore: filestore is any object with the following methods:
-
-            storeFile(self, f)
-                where f is a file instance
-
+        :arg filestore: filestore for temporary files
         :arg show_image_thumbnail: a boolean that, if set, will include an image
             thumbnail with the widget
         :arg css_class: extra css classes to apply to the widget
@@ -426,20 +649,22 @@ class FileUpload(Widget):
         XXX allow_clear -> allow_delete 
         XXX url_ident_factory -> filestore_key_factory
         """
+        # Setup defaults.
+        if filestore is UNSET:
+            filestore = CachedTempFilestore()
+        if url_base is None:
+            url_base = '/filehandler'
+        if url_ident_factory is None:
+            url_ident_factory = lambda i: i.filename
+        # Initialise instance state
         Widget.__init__(self)
         self.filestore = filestore
         self.show_image_thumbnail = show_image_thumbnail
         self.image_thumbnail_default = image_thumbnail_default
-        if url_base is None:
-            self.url_base = '/filehandler'
-        else:
-            self.url_base = url_base
+        self.url_base = url_base
         self.show_download_link = show_download_link
         self.show_file_preview = show_file_preview
-        if url_ident_factory is not None:
-            self.url_ident_factory = url_ident_factory
-        else:
-            self.url_ident_factory = lambda i: i.filename
+        self.url_ident_factory = url_ident_factory
 
     def __repr__(self):
         attributes = []
@@ -462,18 +687,13 @@ class FileUpload(Widget):
             attributes.append('empty=%r'%self.empty)
 
         return 'formish.%s(%s)'%(self.__class__.__name__, ', '.join(attributes))
-          
 
     def urlfactory(self, data):
         if not data:
             return self.image_thumbnail_default
-        if isinstance(data, SchemaFile):
-            key = self.url_ident_factory(data)
-        else:
-            key = data
-        return '%s/%s' % (self.url_base, key)
+        return '%s/%s' % (self.url_base, data)
     
-    def to_request_data(self, schema_type, data):
+    def to_request_data(self, field, data):
         """
         We use the url factory to get an identifier for the file which we use
         as the name. We also store it in the 'default' field so we can check if
@@ -481,20 +701,22 @@ class FileUpload(Widget):
         """
         mimetype = ''
         if isinstance(data, SchemaFile):
-            default = self.url_ident_factory(data)
+            default = util.encode_file_resource_path(None, self.url_ident_factory(data))
             mimetype = data.mimetype
         elif data is not None:
-            default = data
+            default = util.encode_file_resource_path(None, data)
         else:
             default = ''
         return {'name': [default], 'default':[default], 'mimetype':[mimetype]}
     
-    def pre_parse_incoming_request_data(self, schema_type, data, full_request_data):
+    def pre_parse_incoming_request_data(self, field, data):
         """
         File uploads are wierd; in out case this means assymetric. We store the
         file in a temporary location and just store an identifier in the field.
         This at least makes the file look symmetric.
         """
+        if data is None:
+            data = {}
         if data.get('remove', [None])[0] is not None:
             data['name'] = ['']
             data['mimetype'] = ['']
@@ -502,13 +724,18 @@ class FileUpload(Widget):
 
         fieldstorage = data.get('file', [''])[0]
         if getattr(fieldstorage,'file',None):
-            filename = '%s-%s'%(uuid.uuid4().hex,fieldstorage.filename)
-            self.filestore.put(filename, fieldstorage.file, fieldstorage.type, uuid.uuid4().hex)
-            data['name'] = [filename]
+            # XXX Can we reuse the key from a previous temp upload to avoid
+            # creating an additional temp file?
+            key = uuid.uuid4().hex
+            cache_tag = uuid.uuid4().hex
+            self.filestore.put(key, fieldstorage.file, cache_tag,
+                               [('Content-Type', fieldstorage.type),
+                                ('Filename', fieldstorage.filename)])
+            data['name'] = [util.encode_file_resource_path('tmp', key)]
             data['mimetype'] = [fieldstorage.type]
         return data
     
-    def from_request_data(self, schema_type, request_data):
+    def from_request_data(self, field, request_data):
         """
         Creates a File object if possible
         """
@@ -519,12 +746,13 @@ class FileUpload(Widget):
         elif request_data['name'] == request_data['default']:
             return SchemaFile(None, None, None)
         else:
-            filename = request_data['name'][0]
+            key = util.decode_file_resource_path(request_data['name'][0])[1]
             try:
-                content_type, cache_tag, f = self.filestore.get(filename)
+                cache_tag, headers, f = self.filestore.get(key)
             except KeyError:
                 return None
-            return SchemaFile(f, filename, content_type)
+            headers = dict(headers)
+            return SchemaFile(f, headers['Filename'], headers['Content-Type'])
 
     
 class SelectChoice(Widget):
@@ -550,20 +778,16 @@ class SelectChoice(Widget):
         self.options = _normalise_options(options)
 
             
-    def selected(self, option, value, schema_type):
+    def selected(self, option, field):
         """
         Check the value passed matches the actual value
         """
-        if value == '':
-            v = self.empty
-        else:
-            v = value
-        if option[0] == v:
+        if option[0] == field.value[0]:
             return ' selected="selected"'
         else:
             return ''
 
-    def get_options(self, schema_type):
+    def get_options(self, field):
         """
         Return all of the options for the widget
         """
@@ -572,14 +796,14 @@ class SelectChoice(Widget):
             if value == self.empty:
                 options.append( ('',label) )
             else:
-                options.append( (string_converter(schema_type).from_type(value),label) )
+                options.append( (string_converter(field.attr).from_type(value),label) )
         return options
     
-    def get_none_option_value(self, schema_type):
+    def get_none_option_value(self, field):
         """
         Get the default option (the 'unselected' option)
         """
-        none_option =  string_converter(schema_type).from_type(self.none_option[0])
+        none_option =  string_converter(field.attr).from_type(self.none_option[0])
         if none_option is self.empty:
             return ''
         return none_option
@@ -604,6 +828,7 @@ class SelectWithOtherChoice(SelectChoice):
     """
     type = 'SelectWithOtherChoice'
     template = 'field.SelectWithOtherChoice'
+    default_value = {'select': [''], 'other': ['']}
 
     other_option = ('...', 'Other ...')
 
@@ -614,22 +839,24 @@ class SelectWithOtherChoice(SelectChoice):
         self.strip = k.pop('strip',True)
         SelectChoice.__init__(self, options, **k)
 
-    def to_request_data(self, schema_type, data):
+    def to_request_data(self, field, data):
         """
         populate the other choice if needed
         """
-        string_data = string_converter(schema_type).from_type(data)
+        string_data = string_converter(field.attr).from_type(data)
+        if string_data is None:
+            return {'select': [self.get_none_option_value(field)], 'other': ['']}
         if string_data in [value for value, label in self.options]:
-            return {'select': ['...'], 'other': [string_data]}
-        return {'select': [string_data], 'other': ['']}
+            return {'select': [string_data], 'other': ['']}
+        return {'select': [self.other_option[0]], 'other': [string_data]}
 
-    def from_request_data(self, schema_type, request_data):
+    def from_request_data(self, field, request_data):
         """
         Check to see if we need to use the 'other' value
         """
         select = request_data['select'][0]
         other = request_data['other'][0]
-        if select == '...':
+        if select == self.other_option[0]:
             value = other
         else:
             if other != '':
@@ -637,23 +864,15 @@ class SelectWithOtherChoice(SelectChoice):
             value = select
         if value == '':
             return self.empty
-        return string_converter(schema_type).to_type(value)
+        return string_converter(field.attr).to_type(value)
 
-    def get_other_option(self, schema_type):
-        """ Get the other option """
-        return (string_converter(schema_type).from_type( self.other_option[0]), self.other_option[1] )
-            
-    def selected(self, option, value, schema_type):
+    def selected(self, option, field):
         """ Check the value passed matches the actual value """
-        if option[0] == '...' and value not in [value for value, label in self.get_options(schema_type)]:
+        value = field.value['select']
+        if option[0] == self.other_option[0] and value[0] not in [value for value, label in self.get_options(field)]:
             return ' selected="selected"'
-        # Map the empty value
-        if value == '':
-            v = self.empty
-        else:
-            v = value
         # Check for selected
-        if option[0] == v:
+        if option[0] == value[0]:
             return ' selected="selected"'
         else:
             return ''
@@ -682,39 +901,31 @@ class RadioChoice(SelectChoice):
 
     none_option = (None, '- choose -')
 
-    def selected(self, option, value, schema_type):
+    def selected(self, option, field):
         """
         Check if the currently rendering input is the same as the value
         """
-        if value == '':
-            v = self.empty
-        else:
-            v = value
-        if option[0] == v:
+        if option[0] == field.value[0]:
             return ' checked="checked"'
         else:
             return ''
 
-    def from_request_data(self, schema_type, request_data):
+    def from_request_data(self, field, request_data):
         """
         If we don't have a choice, set a blank value
         """
-
-        if not request_data:
-            string_data = ''
-        else:
-            string_data = request_data[0]
+        string_data = request_data[0]
 
         if string_data == '':
             return self.empty
 
-        return string_converter(schema_type).to_type(string_data)
+        return string_converter(field.attr).to_type(string_data)
     
-    def get_none_option_value(self, schema_type):
+    def get_none_option_value(self, field):
         """
         Get the default option (the 'unselected' option)
         """
-        none_option =  string_converter(schema_type).from_type(self.none_option[0])
+        none_option =  string_converter(field.attr).from_type(self.none_option[0])
         if none_option is self.empty:
             return ''
         return none_option
@@ -722,7 +933,7 @@ class RadioChoice(SelectChoice):
     def __repr__(self):
         attributes = []
         attributes.append('options=%r'%self.options)
-        if self.none_option is not UNSET:
+        if self.none_option and self.none_option is not UNSET:
             attributes.append('none_option=%r'%list(self.none_option))
         if self.converter_options != {'delimiter':','}:
             attributes.append('converter_options=%r'%self.converter_options)
@@ -740,36 +951,37 @@ class CheckboxMultiChoice(Widget):
 
     type = 'CheckboxMultiChoice'
     template = 'field.CheckboxMultiChoice'
+    default_value = []
 
     def __init__(self, options, css_class=None):
         self.options = _normalise_options(options)
         Widget.__init__(self, css_class=css_class)
             
-    def to_request_data(self, schema_type, data):
+    def to_request_data(self, field, data):
         """
         Iterate over the data, converting each one
         """
         if data is None: 
             return []
-        return [string_converter(schema_type.attr).from_type(d) for d in data]
+        return [string_converter(field.attr.attr).from_type(d) for d in data]
     
-    def from_request_data(self, schema_type, request_data):
+    def from_request_data(self, field, request_data):
         """
         Iterating to convert back to the source data
         """
-        return [string_converter(schema_type.attr).to_type(d) \
+        return [string_converter(field.attr.attr).to_type(d) \
                 for d in request_data]
 
-    def checked(self, option, values, schema_type):
+    def checked(self, option, field):
         """
         For each value, convert it and check to see if it matches the input data
         """
-        if values is None:
+        if field.value is None:
             return ''
         cvs = []
-        for v in values:
+        for v in field.value:
             try:
-                cvs.append( string_converter(schema_type.attr).to_type(v) )
+                cvs.append( string_converter(field.attr.attr).to_type(v) )
             except ConvertError:
                 continue
         if option[0] in cvs:
@@ -813,24 +1025,25 @@ class CheckboxMultiChoiceTree(Widget):
 
     type = 'CheckboxMultiChoiceTree'
     template = 'field.CheckboxMultiChoiceTree'
+    default_value = []
 
-    def __init__(self, options, cssClass=None):
+    def __init__(self, options, css_class=None):
         self.options = options
         self.optiontree = mktree(options)
-        Widget.__init__(self,cssClass=cssClass)
+        Widget.__init__(self,css_class=css_class)
             
-    def to_request_data(self, schema_type, data):
-        if data is None: 
-            return []
-        return [string_converter(schema_type.attr).from_type(d) for d in data]
+    def to_request_data(self, field, data):
+        return [string_converter(field.attr.attr).from_type(d) for d in data]
     
-    def from_request_data(self, schema_type, data):
-        return [string_converter(schema_type.attr).to_type(d) for d in data]
+    def from_request_data(self, field, request_data):
+        if request_data is None:
+            request_data = []
+        return [string_converter(field.attr.attr).to_type(d) for d in request_data]
 
-    def checked(self, option, values, schema_type):
-        if values is not None:
-            typed_values = self.from_request_data(schema_type,values)
-        if values is not None and option[0] in typed_values:
+    def checked(self, option, field):
+        if field.value is not None:
+            typed_values = self.from_request_data(field.attr,field.value)
+        if field.value is not None and option[0] in typed_values:
             return ' checked="checked"'
         else:
             return ''        
